/*
 * Copyright (c) 2013-2024 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window to view the configured execution history result.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryResult', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.work.ViewTextWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required config property containing the execution history record ID.
   */
  execHistoryId : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.execHistoryId )
      Ext.Error.raise( 'Required config parameter missing: execHistoryId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryResult.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.execHistoryActionResult,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        execHistoryId : me.execHistoryId
      },
      scope : me,
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            me._viewText( json );

            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  },


  _viewText : function ( json )
  {
    var me = this;

    var formattedResult = me._formatResult( json.data );

    var window = Ext.widget( 'ViewTextWindow', {
      cls : 'quartzViewResultWindow',
      iconCls : 'windowIcon',
      title : QDesk.Msg.get(
          'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryResult.windowTitle' )
    } );

    window.on( 'afterrender', function ()
    {
      window.maximize();
      window.toFront( false );
    } );

    var viewPanel = window.down( '#viewPanel' );
    viewPanel.setHtml( formattedResult );

    window.on( 'show', function ()
    {
      /*
       * Set focus on the view panel so that the user can immediately use CTRL+A to
       * select all text in the view panel. The delay is used to defer setting of the
       * focus to avoid a known issue (see http://forums.ext.net/showthread.php?3315-CLOSED-Set-Focus-to-textfield-in-logon-window).
       */
      QDesk.util.UxUtils.focus( viewPanel, false );
    } );

    window.show();
  },


  /**
   * Formats the specified result.
   * @param result the result.
   * @return the formatted result as an HTML string.
   * @private
   */
  _formatResult : function ( result )
  {
    var textColor = QDesk.ApplicationConfig.get( 'viewResult.textColor' );

    var html = '<pre style="margin-bottom: 0; margin-top: 0;">';
    html += '<span style="color: ' + textColor + '">';
    html += Ext.String.htmlEncode( result );
    html += '</span>';
    html += '</pre>';

    return html;
  }
} );
