/*
 * Copyright (c) 2013-2024 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various Quartz calendar-related utility methods.
 */
Ext.define( 'QDesk.util.quartz.QuartzCalendarUtils', {
  singleton : true,

  requires : [
    'QDesk.model.scheduler.quartz.QuartzCalendarType',

    'QDesk.Msg',

    'QDesk.model.common.Month',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils'
  ],


  /**
   * Creates and returns HTML code with calendar details displayed by the rowexpander in the QuartzCalendarsGrid.
   * @param values a calendar value object (not a QuartzCalendar model instance!).
   * @param ownerGrid the owner grid.
   * @returns {string}
   */
  createRowDetailsHtml : function ( values, ownerGrid )
  {
    var html = '<table class="rowDetailsTable">' +
        '<tbody>';

    /* Start of Description */
    var description = Ext.isEmpty( values.description ) ? QDesk.Msg.get( 'QDesk.notAvailable' ) : values.description;

    html += '<tr>';
    html += '<td style="font-weight: bold; width: 100px;">' + QDesk.Msg.get( 'QDesk.description' ) + '</td>';
    html += '<td>' + Ext.String.htmlEncode( description ) + '</td>';
    html += '</tr>';
    /* End of Description */

    html += '</tbody>' +
        '</table>';

    html += '<p/>';
    html += QDesk.util.quartz.QuartzCalendarUtils._createRowDetailsTableHtml( values );

    return html;
  },

  /**
   * Creates a QDesk.model.scheduler.quartz.QuartzCalendar model instance and populates it with
   * the specified EditXXXCalendarForm field values.
   * @param fieldValues a EditXXXTCalendarForm field values.
   * @return the created QDesk.model.scheduler.quartz.QuartzCalendar model instance.
   */
  createCalendar : function ( fieldValues )
  {
    var me = this;

    var calendar = Ext.create( 'QDesk.model.scheduler.quartz.QuartzCalendar' );

    /* Start of common calendar attributes */

    if ( !Ext.isEmpty( fieldValues.type ) )
      calendar.set( 'type', fieldValues.type );

    if ( !Ext.isEmpty( fieldValues.name ) )
      calendar.set( 'name', fieldValues.name );

    if ( !Ext.isEmpty( fieldValues.description ) )
      calendar.set( 'description', fieldValues.description );

    if ( !Ext.isEmpty( fieldValues.baseCalendarName ) )
      calendar.set( 'baseCalendarName', fieldValues.baseCalendarName );

    /* End of common calendar attributes */

    /* Start of calendar-type specific attributes */

    if ( fieldValues.type == QDesk.model.scheduler.quartz.QuartzCalendarType.ANNUAL )
    {
      // TODO: set annual calendar-specific attributes
      // { name : 'yearday', type : 'auto' }, // list of QDesk.model.common.Yearday

      if ( !Ext.isEmpty( fieldValues.timeZone ) )
        calendar.set( 'timeZone', { id : fieldValues.timeZone } );
    }
    else if ( fieldValues.type == QDesk.model.scheduler.quartz.QuartzCalendarType.CRON )
    {
      if ( !Ext.isEmpty( fieldValues.cronExpression ) )
        calendar.set( 'cronExpression', fieldValues.cronExpression );

      if ( !Ext.isEmpty( fieldValues.timeZone ) )
        calendar.set( 'timeZone', { id : fieldValues.timeZone } );
    }
    else if ( fieldValues.type == QDesk.model.scheduler.quartz.QuartzCalendarType.DAILY )
    {
      // TODO: set daily calendar-specific attributes
      // { name : 'timeRangeFrom', type : 'auto', allowNull : true }, // QDesk.model.common.DayTime
      // { name : 'timeRangeTo', type : 'auto', allowNull : true }, // QDesk.model.common.DayTime
      // { name : 'invertTimeRange', type : 'boolean', allowNull : true },

      if ( !Ext.isEmpty( fieldValues.timeZone ) )
        calendar.set( 'timeZone', { id : fieldValues.timeZone } );
    }
    else if ( fieldValues.type == QDesk.model.scheduler.quartz.QuartzCalendarType.HOLIDAY )
    {
      // TODO: set holiday calendar-specific attributes
      // { name : 'date', type : 'auto' }, // list of QDesk.model.common.Date

      if ( !Ext.isEmpty( fieldValues.timeZone ) )
        calendar.set( 'timeZone', { id : fieldValues.timeZone } );
    }
    else if ( fieldValues.type == QDesk.model.scheduler.quartz.QuartzCalendarType.MONTHLY )
    {
      // TODO: set monthly calendar-specific attributes
      // { name : 'day', type : 'auto' }, // list of int

      if ( !Ext.isEmpty( fieldValues.timeZone ) )
        calendar.set( 'timeZone', { id : fieldValues.timeZone } );
    }
    else if ( fieldValues.type == QDesk.model.scheduler.quartz.QuartzCalendarType.WEEKLY )
    {
      // TODO: set weekly calendar-specific attributes
      // { name : 'weekday', type : 'auto' }, // list of QDesk.model.common.WeekDay

      if ( !Ext.isEmpty( fieldValues.timeZone ) )
        calendar.set( 'timeZone', { id : fieldValues.timeZone } );
    }
    else if ( fieldValues.type == QDesk.model.scheduler.quartz.QuartzCalendarType.CUSTOM )
    {
      // TODO: ???
    }
    else
    {
      Ext.Error.raise( 'Unexpected calendar type: ' + fieldValues.type );
    }

    /* End of calendar-type specific attributes */

    return calendar;
  },


  /**
   * Creates and returns an HTML table for the specified calendar object.
   * This method is invoked from the calendars grids to obtain a formatted
   * HTML code to display calendar properties.
   * @param values a calendar value object (not a QuartzCalendar model instance!).
   * @returns {string}
   */
  _createRowDetailsTableHtml : function ( values )
  {
    var rows = [];
    switch ( values.type )
    {
      case QDesk.model.scheduler.quartz.QuartzCalendarType.ANNUAL:
        rows.push( [
          QDesk.Msg.get( 'QDesk.util.quartz.QuartzCalendarUtils.rowDetailsTable.rowYearDays' ),
          QDesk.util.quartz.QuartzCalendarUtils._formatYearDays( values.yearday )
        ] );

        if ( values.timeZone != null )
        {
          rows.push( [
                QDesk.Msg.get( 'QDesk.util.quartz.QuartzCalendarUtils.rowDetailsTable.rowTimeZone' ),
                values.timeZone.displayName
              ]
          );
        }
        return QDesk.util.CommonUtils.createBasicPropertiesTable( rows );

      case QDesk.model.scheduler.quartz.QuartzCalendarType.CRON:
        rows.push( [
          QDesk.Msg.get( 'QDesk.util.quartz.QuartzCalendarUtils.rowDetailsTable.rowCronExpression' ),
          values.cronExpression
        ] );

        if ( values.timeZone != null )
        {
          rows.push( [
                QDesk.Msg.get( 'QDesk.util.quartz.QuartzCalendarUtils.rowDetailsTable.rowTimeZone' ),
                values.timeZone.displayName
              ]
          );
        }
        return QDesk.util.CommonUtils.createBasicPropertiesTable( rows );

      case QDesk.model.scheduler.quartz.QuartzCalendarType.DAILY:
        rows.push( [
          QDesk.Msg.get( 'QDesk.util.quartz.QuartzCalendarUtils.rowDetailsTable.rowTimeRange' ),
          QDesk.util.quartz.QuartzCalendarUtils._formatTimeRange( values.timeRangeFrom, values.timeRangeTo,
              values.invertTimeRange )
        ] );

        if ( values.timeZone != null )
        {
          rows.push( [
                QDesk.Msg.get( 'QDesk.util.quartz.QuartzCalendarUtils.rowDetailsTable.rowTimeZone' ),
                values.timeZone.displayName
              ]
          );
        }
        return QDesk.util.CommonUtils.createBasicPropertiesTable( rows );

      case QDesk.model.scheduler.quartz.QuartzCalendarType.HOLIDAY:
        rows.push( [
          QDesk.Msg.get( 'QDesk.util.quartz.QuartzCalendarUtils.rowDetailsTable.rowDates' ),
          QDesk.util.quartz.QuartzCalendarUtils._formatDates( values.date )
        ] );

        if ( values.timeZone != null )
        {
          rows.push( [
                QDesk.Msg.get( 'QDesk.util.quartz.QuartzCalendarUtils.rowDetailsTable.rowTimeZone' ),
                values.timeZone.displayName
              ]
          );
        }
        return QDesk.util.CommonUtils.createBasicPropertiesTable( rows );

      case QDesk.model.scheduler.quartz.QuartzCalendarType.MONTHLY:
        rows.push( [
          QDesk.Msg.get( 'QDesk.util.quartz.QuartzCalendarUtils.rowDetailsTable.rowDays' ),
          QDesk.util.quartz.QuartzCalendarUtils._formatDays( values.day )
        ] );

        if ( values.timeZone != null )
        {
          rows.push( [
                QDesk.Msg.get( 'QDesk.util.quartz.QuartzCalendarUtils.rowDetailsTable.rowTimeZone' ),
                values.timeZone.displayName
              ]
          );
        }
        return QDesk.util.CommonUtils.createBasicPropertiesTable( rows );

      case QDesk.model.scheduler.quartz.QuartzCalendarType.WEEKLY:
        rows.push( [
          QDesk.Msg.get( 'QDesk.util.quartz.QuartzCalendarUtils.rowDetailsTable.rowWeekDays' ),
          QDesk.util.quartz.QuartzCalendarUtils._formatWeekDays( values.weekday )
        ] );

        if ( values.timeZone != null )
        {
          rows.push( [
                QDesk.Msg.get( 'QDesk.util.quartz.QuartzCalendarUtils.rowDetailsTable.rowTimeZone' ),
                values.timeZone.displayName
              ]
          );
        }
        return QDesk.util.CommonUtils.createBasicPropertiesTable( rows );

      case QDesk.model.scheduler.quartz.QuartzCalendarType.CUSTOM:
        rows.push( [
          QDesk.Msg.get( 'QDesk.util.quartz.QuartzCalendarUtils.rowDetailsTable.rowClassName' ),
          values.className
        ] );

        if ( values.timeZone != null )
        {
          rows.push( [
                QDesk.Msg.get( 'QDesk.util.quartz.QuartzCalendarUtils.rowDetailsTable.rowTimeZone' ),
                values.timeZone.displayName
              ]
          );
        }
        return QDesk.util.CommonUtils.createBasicPropertiesTable( rows );

      default:
        return '';
    }
  },


  _formatTimeRange : function ( timeRangeFrom, timeRangeTo, invertTimeRange )
  {
    var result = '';

    if ( invertTimeRange )
    {
      result += '00:00:00-' + timeRangeFrom;
      result += ' ';
      result += timeRangeTo + '-00:00:00';
    }
    else
    {
      result += timeRangeFrom + '-' + timeRangeTo;
    }
    return result;
  },


  _formatDates : function ( dates )
  {
    if ( Ext.isEmpty( dates ) )
    {
      return '';
    }
    else
    {
      var result = '';
      Ext.each( dates, function ( date, index )
      {
        var months = [
          QDesk.model.common.Month.JANUARY,
          QDesk.model.common.Month.FEBRUARY,
          QDesk.model.common.Month.MARCH,
          QDesk.model.common.Month.APRIL,
          QDesk.model.common.Month.MAY,
          QDesk.model.common.Month.JUNE,
          QDesk.model.common.Month.JULY,
          QDesk.model.common.Month.AUGUST,
          QDesk.model.common.Month.SEPTEMBER,
          QDesk.model.common.Month.OCTOBER,
          QDesk.model.common.Month.NOVEMBER,
          QDesk.model.common.Month.DECEMBER
        ];

        // date.month
        var month = months.indexOf( date.month );
        if ( month == -1 )
        {
          // skip the date
        }
        else
        {
          var jsDate = new Date( date.year, month, date.day );
          result += QDesk.util.FormatUtils.formatDate( jsDate );
          if ( index < dates.length - 1 )
            result += '<br/>';
        }
      } );
      return result;
    }
  },


  _formatDays : function ( days )
  {
    if ( Ext.isEmpty( days ) )
    {
      return '';
    }
    else
    {
      var result = '';
      Ext.each( days, function ( day, index )
      {
        result += day;
        if ( index < days.length - 1 )
          result += '<br/>';
      } );
      return result;
    }
  },


  _formatYearDays : function ( yeardays )
  {
    if ( Ext.isEmpty( yeardays ) )
    {
      return '';
    }
    else
    {
      var result = '';
      Ext.each( yeardays, function ( yearday, index )
      {
        result += QDesk.util.EnumUtils.getMonthName( yearday.month ) + ' ' + yearday.day;
        if ( index < yeardays.length - 1 )
          result += '<br/>';
      } );
      return result;
    }
  },


  _formatWeekDays : function ( weekdays )
  {
    if ( Ext.isEmpty( weekdays ) )
    {
      return '';
    }
    else
    {
      var result = '';
      Ext.each( weekdays, function ( weekday, index )
      {
        result += QDesk.util.EnumUtils.getWeekdayName( weekday );
        if ( index < weekdays.length - 1 )
          result += '<br/>';
      } );
      return result;
    }
  }
} );
