/*
 * Copyright (c) 2013-2024 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.store.scheduler.quartz.QuartzCalendarStore',
    'QDesk.util.constant.DateFormat',
    'QDesk.util.EnumUtils',

    'QDesk.util.quartz.QuartzCalendarUtils',

    'QDesk.view.widget.WSearchField',
    'QDesk.view.widget.WReadOnlyCheckColumn',

    'QDesk.view.work.scheduler.quartz.calendar.CalendarsGridController',

    'Ext.grid.plugin.RowExpander'
  ],

  alias : 'widget.QuartzCalendarsGrid',

  border : false,
  scrollable : true,

  listeners : {
    itemcontextmenu : function ( grid, record, item, index, event )
    {
      var me = this;
      event.stopEvent();
      var rowCtxMenu = me._createRowCtxMenu( record );
      rowCtxMenu.showAt( event.getXY() );
    }
  },

  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '{[QDesk.util.quartz.QuartzCalendarUtils.createRowDetailsHtml( values, this.owner.grid )]}'
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    if ( !config.qdesk.gridId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.gridId' );

    var connectionCtx = config.qdesk.connectionCtx;
    // var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    // var schedulerInfo = connectionCtx.getSchedulerInfo();

    // var schedulerVersion = schedulerInfo.getSchedulerVersion();
    // var jvmAgentReady = jvmAgentInfo.isReady();
    // var publicApiReady = schedulerInfo.isPublicApiReady();

    Ext.apply( config, {
      controller : Ext.create(
          'QDesk.view.work.scheduler.quartz.calendar.CalendarsGridController', {
            qdesk : {
              connectionCtx : connectionCtx
            }
          } )
    } );

    var columns = [];

    columns.push(
        {
          itemId : 'nameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.textNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.tooltipNameColumn' ),
          dataIndex : 'name',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },

        {
          itemId : 'typeColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.typeColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.tooltipTypeColumn' ),
          dataIndex : 'type',
          sortable : true,
          groupable : false,
          flex : 0.5,
          renderer : function ( value )
          {
            return QDesk.util.quartz.QuartzEnumUtils.getCalendarTypeName( value );
          }
        },

        {
          itemId : 'baseCalendarNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.textBaseCalendarNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.tooltipBaseCalendarNameColumn' ),
          dataIndex : 'baseCalendarName',
          sortable : false,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        }
    );

    var store = Ext.create( 'QDesk.store.scheduler.quartz.QuartzCalendarStore' );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                me.fireEvent( 'refresh' );
              }
            }
          }
        },
        //'-',
        {
          xtype : 'button',
          itemId : 'addButton',
          scale : 'small',
          iconCls : 'addCalendar',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.tooltipAddCalendarButton' ),
          disabled : true,
          hidden : true, // TODO: unhide when the functionality has been implemented
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_CALENDAR_ADD ],
          qdNotAuthorizedIconCls : 'addCalendarNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                me.fireEvent( 'addCalendar' );
              }
            }
          }
        },
        {
          xtype : 'button',
          itemId : 'deleteButton',
          scale : 'small',
          iconCls : 'deleteCalendar',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.tooltipDeleteCalendarButton' ),
          disabled : true,
          hidden : true, // TODO: unhide when the functionality has been implemented
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_CALENDAR_DELETE ],
          qdNotAuthorizedIconCls : 'deleteCalendarNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                var calendar = me.getSelectionModel().getSelection()[ 0 ];
                me.fireEvent( 'deleteCalendar', calendar );
              }
            }
          }
        },
        {
          xtype : 'button',
          itemId : 'editButton',
          scale : 'small',
          iconCls : 'editCalendar',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.tooltipEditCalendarButton' ),
          disabled : true,
          hidden : true, // TODO: unhide when the functionality has been implemented
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_CALENDAR_UPDATE ],
          qdNotAuthorizedIconCls : 'editCalendarNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                var calendar = me.getSelectionModel().getSelection()[ 0 ];
                me.fireEvent( 'editCalendar', calendar );
              }
            }
          }
        },
        {
          xtype : 'button',
          itemId : 'cloneButton',
          scale : 'small',
          iconCls : 'cloneCalendar',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.tooltipCloneCalendarButton' ),
          disabled : true,
          hidden : true, // TODO: unhide when the functionality has been implemented
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_CALENDAR_ADD ],
          qdNotAuthorizedIconCls : 'cloneCalendarNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                var calendar = me.getSelectionModel().getSelection()[ 0 ];
                me.fireEvent( 'cloneCalendar', calendar );
              }
            }
          }
        },
        //'-',
        {
          xtype : 'button',
          itemId : 'viewAuditTrailButton',
          scale : 'small',
          iconCls : 'viewCalendarAuditTrail',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.tooltipViewCalendarAuditTrailButton' ),
          disabled : true,
          hidden : true, // TODO: unhide when the functionality has been implemented
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.AUDIT_TRAIL_VIEW ],
          qdNotAuthorizedIconCls : 'viewCalendarAuditTrailNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                var calendar = me.getSelectionModel().getSelection()[ 0 ];
                me.fireEvent( 'viewCalendarAuditTrail', calendar );
              }
            }
          }
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
      ],


      columns : columns,

      store : store,

      qdesk : {
        connectionCtx : connectionCtx,
        gridId : config.qdesk.gridId
      }
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    me.getStore().on( 'load', function ( store, records, successful, operation )
    {
      var view = me.getView();

      if ( successful )
      {
        if ( Ext.isEmpty( records ) )
        {
          view.emptyText = QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.emptyTextNoData' );

          view.refresh();
        }
      }
      else
      {
        view.emptyText =
            QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.emptyTextErrorLoadingData' );

        view.refresh();
      }
    } );
  },


  /**
   * Invoked to create a context menu for the specified calendar.
   * @param calendar a calendar.
   * @private
   */
  _createRowCtxMenu : function ( calendar )
  {
    var me = this;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // delete calendar
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.ruleCtxMenuDelete' ),
          iconCls : 'deleteCalendar',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_CALENDAR_DELETE ],
          qdNotAuthorizedIconCls : 'deleteCalendarNAUTH',
          handler : function ()
          {
            me.fireEvent( 'deleteCalendar', calendar );
          }
        },
        // edit calendar
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.ruleCtxMenuEdit' ),
          iconCls : 'editCalendar',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_CALENDAR_UPDATE ],
          qdNotAuthorizedIconCls : 'editCalendarNAUTH',
          handler : function ()
          {
            me.fireEvent( 'editCalendar', calendar );
          }
        },
        // clone calendar
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.ruleCtxMenuClone' ),
          iconCls : 'cloneCalendar',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_CALENDAR_ADD ],
          qdNotAuthorizedIconCls : 'cloneCalendarNAUTH',
          handler : function ()
          {
            me.fireEvent( 'cloneCalendar', calendar );
          }
        },
        '-',
        // view audit trail
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.calendar.CalendarsGrid.ruleCtxMenuViewAuditTrail' ),
          iconCls : 'viewCalendarAuditTrail',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.AUDIT_TRAIL_VIEW ],
          qdNotAuthorizedIconCls : 'viewCalendarAuditTrailNAUTH',
          handler : function ()
          {
            me.fireEvent( 'viewCalendarAuditTrail', calendar );
          }
        }
      ]
    } );
  }
} );
