/*
 * Copyright (c) 2013-2024 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.calendar.EditCalendarBaseForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.store.scheduler.quartz.QuartzCalendarNameStore',

    'QDesk.util.quartz.QuartzUtils',
    'QDesk.util.quartz.QuartzEnumUtils',

    'QDesk.view.work.scheduler.quartz.calendar.EditCalendarFormMode'
  ],

  bodyPadding : '5 10 10 10',

  fieldDefaults : {
    labelWidth : 100
  },

  defaultButton : 'actionButton',

  qdesk : {
    /**
     * Required {@link QDesk.ConnectionContext} instance.
     */
    connectionCtx : undefined,

    /*
     * Mode of the form (see EditCalendarFormMode).
     */
    mode : undefined,

    /**
     * Calendar type (see QDesk.model.scheduler.quartz.QuartzCalendarType).
     */
    calendarType : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.calendarType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.calendarType' );

    var connectionCtx = config.qdesk.connectionCtx;
    var jvmAgentReady = connectionCtx.getJvmAgentInfo().isReady();
    var publicApiReady = connectionCtx.getSchedulerInfo().isPublicApiReady();

    var propertiesItems = me._getCalendarSpecificFormItems( config.qdesk );

    var baseCalendarNameStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzCalendarNameStore', {
      autoLoad : false
    } );

    baseCalendarNameStore.getProxy().extraParams = {
      connectionId : connectionCtx.getConnectionId()
    };

    Ext.apply( config, {
      items : [
        /* Start of hidden fields */
        {
          xtype : 'hiddenfield',
          itemId : 'type',
          name : 'type',
          value : config.qdesk.calendarType
        },
        /* End of hidden fields */

        {
          xtype : 'displayfield',
          cls : 'formHeadingText',
          value : me._getCalendarSpecificFormHeadingText( config.qdesk.calendarType ),
          listeners : {
            render : function ( c )
            {
              var field = c.getEl().select( 'div.x-form-display-field' );
              if ( field )
              {
                var tooltip = me._getCalendarSpecificFormHeadingTooltip( config.qdesk.calendarType );
                field.createChild( {
                  tag : 'img',
                  src : 'images/icon/tooltip-10x10.png',
                  style : 'margin-bottom: -1px; margin-left: 5px; padding: 0px; width: 10px; height: 10px;',
                  'data-qtip' : Ext.String.htmlEncode( tooltip )
                } );
              }
            }
          }
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.calendar.EditCalendarBaseForm.fieldsetLabelTrigger' ),
          padding : '0 10 0 10',
          defaults : {
            labelWidth : 115,
            anchor : '100%'
          },
          items : [
            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              labelWidth : 55,
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.calendar.EditCalendarBaseForm.fieldLabelName' ),
              fieldTooltip : config.qdesk.mode == QDesk.view.work.scheduler.quartz.calendar.EditCalendarFormMode.EDIT ?
                  null :
                  QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.calendar.EditCalendarBaseForm.fieldTooltipName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.calendar.EditCalendarBaseForm.fieldEmptyTxtName' ),
              allowBlank : false,
              maxLength : 200,
              readOnly : config.qdesk.mode == QDesk.view.work.scheduler.quartz.calendar.EditCalendarFormMode.EDIT
            },

            {
              xtype : 'textareafield',
              itemId : 'description',
              name : 'description',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.calendar.EditCalendarBaseForm.fieldLabelDescription' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.calendar.EditCalendarBaseForm.fieldTooltipDescription' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.calendar.EditCalendarBaseForm.fieldEmptyTxtDescription' ),
              labelAlign : 'top',
              allowBlank : true,
              maxLength : 250
            },

            {
              xtype : 'xcombo',
              itemId : 'baseCalendarName',
              name : 'baseCalendarName',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.calendar.EditCalendarBaseForm.fieldLabelCalendarName' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.calendar.EditCalendarBaseForm.fieldTooltipCalendarName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.calendar.EditCalendarBaseForm.fieldEmptyTxtCalendarName' ),
              allowBlank : true,
              editable : false,
              forceSelection : false,
              queryMode : 'local',
              store : baseCalendarNameStore,
              valueField : 'name',
              displayField : 'name',
              tpl : Ext.create( 'Ext.XTemplate',
                  '<ul class="x-list-plain"><tpl for=".">',
                  '<li role="option" class="x-boundlist-item">{name:htmlEncode}</li>',
                  '</tpl></ul>' )
            },

              /* Calendar type specific fields */
            {
              xtype : 'container',
              items : propertiesItems
            }
          ]
        }  // end of Calendar fieldset
      ],

      buttons : [
        {
          itemId : 'cancelButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
        },
        {
          itemId : 'actionButton',
          reference : 'actionButton',
          text : me._getActionButtonText( config.qdesk.mode ),
          formBind : jvmAgentReady && publicApiReady,
          disabled : !jvmAgentReady || !publicApiReady,
          qdRequiresAnyPermissions : me._getRequiredPermissions( config.qdesk.mode )
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();

    me._setDefaultFormValues();
  },


  /**
   * Returns an array of all form stores that must be loaded before this form can be opened.
   */
  getFormStores : function ()
  {
    var me = this;

    var stores = [ me.down( '#baseCalendarName' ).getStore() ];

    // append calendar type specific stores
    return stores.concat( me._getCalendarSpecificStores() );
  },


  /**
   * Loads the specified calendar record ({@link QuartzCalendar} instance) into this form.
   * @param calendar the calendar record.
   */
  loadRecord : function ( calendar )
  {
    var me = this;
    me.callParent( arguments );

    me.getForm().setValues( {
      name : calendar.get( 'name' ),
      description : calendar.get( 'description' ),
      baseCalendarName : calendar.get( 'baseCalendarName' )
    } );
  },


  /**
   * Returns the {@link QuartzTrigger} instance with data from this form.
   */
  getCalendarRecord : function ()
  {
    var me = this;

    var form = me.getForm();

    return QDesk.util.quartz.QuartzCalendarUtils.createCalendar( form.getFieldValues() );
  },


  /**
   * Binds event handlers to all form widgets. Sub-classes must not override this method, they
   * should override the _bindCalendarSpecificEventHandlers method.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    // nothing to bind here for generic calendar form fields
    me._bindCalendarSpecificEventHandlers();
  },


  /**
   * Sets the default values of individual form fields. Sub-classes must not override this method, they
   * should override the _setCalendarSpecificFormDefaultValues method to set calendar type specific default
   * form values.
   */
  _setDefaultFormValues : function ()
  {
    var me = this;
    me._setCalendarSpecificFormDefaultValues();
  },


  /**
   * Returns the array of calendar type specific form items. Sub-classes must
   * override this method.
   * @param qdesk a QuartzDesk object wrapping various parameters passed to the form upon its creation.
   * @returns {Array} the array of calendar type specific form items.
   * @private
   */
  _getCalendarSpecificFormItems : function ( qdesk )
  {
    Ext.Error.raise( 'Sub-classes must override the _getCalendarSpecificFormItems method.' );
  },


  /**
   * Binds event handlers to calendar type specific form widgets. Sub-classes can override
   * this method.
   */
  _bindCalendarSpecificEventHandlers : function ()
  {
  },


  /**
   * Sets the default values of individual form fields. Sub-classes can override
   * this method.
   */
  _setCalendarSpecificFormDefaultValues : function ()
  {
  },


  /**
   * Invoked by the base form to obtain the array of all calendar type specific stores from
   * sub-classes. Sub-classes can override this method.
   */
  _getCalendarSpecificStores : function ()
  {
    return [];
  },


  _getCalendarSpecificFormHeadingText : function ( calendarType )
  {
    return QDesk.util.quartz.QuartzEnumUtils.getCalendarTypeName( calendarType );
  },


  _getCalendarSpecificFormHeadingTooltip : function ( calendarType )
  {
    return QDesk.Msg.get(
        'QDesk.view.work.scheduler.quartz.calendar.EditCalendarBaseForm.titleTooltip.' + calendarType );
  },


  _getActionButtonText : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.scheduler.quartz.calendar.EditCalendarFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.buttonLabelAdd' );

      case QDesk.view.work.scheduler.quartz.calendar.EditCalendarFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.buttonLabelUpdate' );

      case QDesk.view.work.scheduler.quartz.calendar.EditCalendarFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.buttonLabelClone' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
        return null;
    }
  },


  _getRequiredPermissions : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.scheduler.quartz.calendar.EditCalendarFormMode.ADD:
        return [ QDesk.model.security.Permission.QUARTZ_CALENDAR_ADD ];

      case QDesk.view.work.scheduler.quartz.calendar.EditCalendarFormMode.EDIT:
        return [ QDesk.model.security.Permission.QUARTZ_CALENDAR_UPDATE ];

      case QDesk.view.work.scheduler.quartz.calendar.EditCalendarFormMode.CLONE:
        return [ QDesk.model.security.Permission.QUARTZ_CALENDAR_ADD ];

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
        return null;
    }
  }
} );
